/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This examples demonstrates how to build PDUs for encoding. It exersizes
 * various IAAPIs needed to construct PDUs for encoding using SEQUENCE OF as
 * a PDU (see test2.asn).
 */


#include "test2.h"
#include "iaapi.h"
 




int main(void)
{

    TypeHndl pduth, sth, oth, bth, softh;
    unsigned int sx, ox, bx, sofx;
    IAAPI_ERRTYPE err;
    int ret;
    void *seqcomp, *octetcomp, *bitcomp, *seqofcomp;
    OssGlobal w, *world = &w;
    OssBuf encodedData = {0};
    PDU *myPDU, *decodedData;
    int pdu_num = 0;


    ret = ossinit(world, test2);
    if (ret)
	printf("Initialization failed with error = %d\n", ret);

    ossSetIaapiErrorHandlingFunction(world, ossDefaultIAAPI_ERR);

	/*
	 * Get the type handle of the CHOICE PDU.
	 */
    pduth = ossTypeHandleOfPDU(world, PDU_PDU);
    ossPrint(world, "\tPDU ASN.1 notation...\n\n");
    ossPrintASN1DescriptionOfType(world, pduth, FALSE);

    ossPrint(world, "\n\tUnencoded PDU from the value notation...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, &pdu1);
    if (ret) {
	ossterm(world);
	return ret;
    }

    ossPrint(world, "\n\tUnencoded PDU from the value notation...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, &pdu2);
    if (ret) {
	ossterm(world);
	return ret;
    }

    ossPrint(world, "\nConstructing the PDU with IAAPI...\n");

	/*
	 * Get the index and the type handle of the SEQUENCE component of
	 * the CHOICE PDU.
	 */
    sx = ossItemIndexByName(world, pduth, "pdu1");
    sth = ossTypeHandleOfComponent(world, pduth, sx);

	/*
	 * Get the index and the type handle of the OCTET STRING component of
	 * the SEQUENCE component of the CHOICE PDU.
	 */
    ox = ossItemIndexByName(world, sth, "data1");
    oth = ossTypeHandleOfComponent(world, sth, ox);
	/*
	 * Create an OCTET STRING component of the SEQUENCE component of
	 * the CHOICE PDU.
	 */
    octetcomp = NULL;
    err = ossPutSimpleTypeValue(world, oth, "'FFFFFF'H", &octetcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a SEQUENCE component of the CHOICE PDU with the
	 * OCTET STRING component.
	 */
    seqcomp = NULL;
    err = ossPutStructTypeValue(world, sth, octetcomp, ox, (void**) &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Get the index and the type handle of the BIT STRING component of
	 * the SEQUENCE component of the CHOICE PDU.
	 */
    bx = ossItemIndexByName(world, sth, "data2");
    bth = ossTypeHandleOfComponent(world, sth, bx);
	/*
	 * Create a BIT STRING component of the SEQUENCE component of
	 * the CHOICE PDU.
	 */
    err = ossPutSimpleTypeValue(world, bth, "'11111'B", &bitcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Add the BIT STRING component to the SEQUENCE component of
	 * the CHOICE PDU.
	 */
    err = ossPutStructTypeValue(world, sth, bitcomp, bx, &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a CHOICE PDU with the SEQUENCE component just constructed.
	 */
    myPDU = NULL;
    err = ossPutStructTypeValue(world, pduth, seqcomp, sx, (void**)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Here we call ossPrintPDU() but it is also possible to call
	 * ossPrintDecodedValueOfPDU().
	 */
    ossPrint(world, "\n\tPDU constructed with IAAPI...\n\n");
    ret = ossPrintPDU(world, PDU_PDU, myPDU);
    if (err) {
	ossterm(world);
	return err;
    }

    ossSetEncodingRules(world, OSS_BER);
    ossSetFlags(world, DEBUGPDU);
    ossPrint(world, "\nEncoding the constructed PDU...\n\n");
    if (ret = ossEncode(world, PDU_PDU, myPDU, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));
    else {
        ossPrint(world, "\n\tEncoded constructed PDU...\n\n");
        ossPrintHex(world, encodedData.value, encodedData.length);
	decodedData = NULL;
	ossPrint(world, "\nDecoding the PDU...\n\n");
	if (ret = ossDecode(world, &pdu_num, &encodedData, (void **)&decodedData))
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decodedData);
	    ossFreePDU(world, pdu_num, decodedData);
	}
	ossFreeBuf(world, encodedData.value);
    }

    ossFreePDU(world, PDU_PDU, myPDU);

    ossPrint(world, "\n\nConstructing another PDU with IAAPI...\n");

	/*
	 * Get the index and the type handle of the SEQUENCE OF
	 * component of the CHOICE PDU.
	 */
    sofx = ossItemIndexByName(world, pduth, "pdu2");
    softh = ossTypeHandleOfComponent(world, pduth, sofx);

	/*
	 * Get the type handle of the SEQUENCE component of the SEQUENCE OF
	 * component of the CHOICE PDU. The last index parameter is ignored
	 * in case of SEQUENCE OF/SET OF.
	 */
    sth = ossTypeHandleOfComponent(world, softh, 0);
	/*
	 * Get the index and the type handle of the OCTET STRING
	 * component of the SEQUENCE component of the SEQUENCE OF
	 * component of the CHOICE PDU.
	 */
    ox = ossItemIndexByName(world, sth, "data1");
    oth = ossTypeHandleOfComponent(world, sth, ox);
	/*
	 * Create an OCTET STRING component of the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    octetcomp = NULL;
    err = ossPutSimpleTypeValue(world, oth, "'222222'H", &octetcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a SEQUENCE component of the SEQUENCE OF component
	 * of the CHOICE PDU with the OCTET STRING component.
	 */
    seqcomp = NULL;
    err = ossPutStructTypeValue(world, sth, octetcomp, ox, &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Get the index and the type handle of the BIT STRING
	 * component of the SEQUENCE component of the SEQUENCE OF
	 * component of the CHOICE PDU.
	 */
    bx = ossItemIndexByName(world, sth, "data2");
    bth = ossTypeHandleOfComponent(world, sth, bx);
	/*
	 * Create a BIT STRING component of the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    err = ossPutSimpleTypeValue(world, bth, "'11111'B", &bitcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Add the BIT STRING component to the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    err = ossPutStructTypeValue(world, sth, bitcomp, bx, &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create SEQUENCE OF component #0 of the CHOICE PDU with
	 * the SEQUENCE component just constructed.
	 */
    seqofcomp = NULL;
    err = ossPutStructTypeValue(world, softh, seqcomp, 0, &seqofcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create an OCTET STRING component of the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    octetcomp = NULL;
    err = ossPutSimpleTypeValue(world, oth, "'222222'H", &octetcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a SEQUENCE component of the SEQUENCE OF component
	 * of the CHOICE PDU with the OCTET STRING component.
	 */
    seqcomp = NULL;
    err = ossPutStructTypeValue(world, sth, octetcomp, ox, &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a BIT STRING component of the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    err = ossPutSimpleTypeValue(world, bth, "'11111'B", &bitcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Add the BIT STRING component to the SEQUENCE component of
	 * the SEQUENCE OF component of the CHOICE PDU.
	 */
    err = ossPutStructTypeValue(world, sth, bitcomp, bx, &seqcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create SEQUENCE OF component #1 of the CHOICE PDU with
	 * the SEQUENCE component just constructed.
	 */
    err = ossPutStructTypeValue(world, softh, seqcomp, 1, &seqofcomp);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Create a CHOICE PDU with the SEQUENCE OF component just constructed.
	 */
    myPDU = NULL;
    err = ossPutStructTypeValue(world, pduth, seqofcomp, sofx, (void**)&myPDU);
    if (err) {
	ossterm(world);
	return err;
    }
	/*
	 * Here we call ossPrintDecodedValueOfPDU() but it is also
	 * possible to call ossPrintPDU().
	 */
    ossPrint(world, "\n\tAnother PDU constructed with IAAPI...\n\n");
    ossPrintDecodedValueOfPDU(world, PDU_PDU, myPDU);

    ossPrint(world, "\nEncoding the constructed PDU...\n\n");
    encodedData.length = 0;
    encodedData.value = NULL;
    if (ret = ossEncode(world, PDU_PDU, myPDU, &encodedData))
        ossPrint(world, "%s\n", ossGetErrMsg(world));
    else {
        ossPrint(world, "\n\tEncoded constructed PDU...\n\n");
        ossPrintHex(world, encodedData.value, encodedData.length);
	decodedData = NULL;
	ossPrint(world, "\nDecoding the PDU...\n\n");
	if (ret = ossDecode(world, &pdu_num, &encodedData, (void **)&decodedData))
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	else {
	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, decodedData);
	    ossFreePDU(world, pdu_num, decodedData);
	}
	ossFreeBuf(world, encodedData.value);
    }
	/*
	 * Here we call ossFreeDecodedValue() but it is also possible
	 * to call ossFreePDU().
	 */
    err = ossFreeDecodedValue(world, pduth, myPDU);

    ossterm(world);
    return ret;
}
